#include "global.h"
#include "x24x_inc.h"

#define NB_RECORDS 1000

/* Tableaux permettant de conserver toutes les valeurs reues lors du
   suivi des couleurs avec la camra. Ces informations sont utiles
   pour le dboggage. */
int camera_barycentre_x[NB_RECORDS];
int camera_barycentre_y[NB_RECORDS];
int camera_confiance[NB_RECORDS];
int camera_current_record;

/* Fichier contenant la gestion de la camera */

char serial_buffer[64];
int serial_buffer_rx_ptr;

/* Variable permettant de signaler au programme principal la fin de 
 * rception du message sur le port srie. Le programme principal 
 * la met  0, envoie la commande sur le port srie. La camra va 
 * alors envoyer ses informations, qui vont tre lues par le handler
 * d'interruption. Lorsque celui-ci rencontrera ':', c'est  dire la
 * fin de la commande, il mettra cette variable  1 pour signaler 
 * au programme principal que la commande est reue
 */
int receptionCommandeTerminee = 0;

/* Comptage des interruptions, pour dbugger */
int nbInterruptionSerie = 0;   

/* Rinitialise ce qu'il faut pour relancer une nouvelle commande */
void serial_init_command(void)
{ 
  int i;

  /* On remet cette valeur a 0 pour pouvoir etre signale de la fin de la commande */
  receptionCommandeTerminee = 0;
  
  /* Rinitialisation du buffer de rception */
  serial_buffer_rx_ptr = 0;
  for (i = 0 ; i < 64 ; i++)
  {
  	serial_buffer[i] = 0;
  }
}
                                       
/* Fonction d'initialisation du port serie */              
void serial_port_init(void)
{ 
  int i;
    
  /* Activation de l'horloge pour le port srie */
  SCSR1 = SCSR1 | 0x40;
  
  /* Active les entrees / sorties pour le port srie (0003) */
  MCRA = MCRA | 0x0003;
  
  /* Initialise les indicateurs (reset) */
  SCICTL1 = 0x0;
  
  /* Active les lignes RX et TX (RX Enable + TX Enable + SW Reset) */
  SCICTL1 = 0x23;                                             
  
  /* Format des donnes (1 bit de stop, pas de parite, 8 bits de donnes) */
  SCICCR = 0x7;
  
  /* Configuration de la vitesse (vitesse = 19 200 => BRR = 0x103) */
  SCIHBAUD = 0x0;
  SCILBAUD = 0xBF;
  
  /* Active les interruptions */
  SCICTL2 = 0x2;
  
  SCIPRI = 0x18;        
  
  serial_init_command();
  
  asm("	SETC INTM");
  IFR = 0xFFFF;
  IMR = IMR | 0x1;
  asm("	CLRC INTM");
  
  for (i = 0; i < NB_RECORDS ; i++)
  {
    camera_barycentre_x[i] = 0;
    camera_barycentre_y[i] = 0;
    camera_confiance[i] = 0;
  }

  camera_current_record = 0;
}             

/* Envoi d'un caractere sur le port serie */
static void serial_port_send(unsigned char c)
{                              
  /* Attente du vidage du buffer de transmission */
  while((SCICTL2 & 0x80) == 0);
  
  /* Envoi */
  SCITXBUF = c;
}

/* Reception d'un caractere du port serie */
static unsigned char serial_port_receive(void)
{              
  /* Attente de la disponibilit d'une donne dans le buffer de rception. 
   * Cette attente n'est plus necessaire, car nous appelons cette fonction 
   * quand l'interruption serie a ete levee, c'est a dire quand une donnee
   * est disponible */
  /* while((SCIRXST & 0x40) == 0); */
  
  /* Rception */
  return (SCIRXBUF & 0xFF);
}

/* Change le mode de la LED de la camra
 *  0: LED teinte
 *  1: LED allume
 *  2: LED automatique (allume en TC, teinte sinon)
 */
void camera_led_change_mode(int mode)
{   
    serial_init_command();    
	mode = mode + '0';
    serial_port_send('L');
    serial_port_send('1');
    serial_port_send(' ');
    serial_port_send(mode);
    serial_port_send('\r');
}

void camera_pm_mode(int mode)
{   
    serial_init_command();
	mode = mode + '0';
    serial_port_send('P');
    serial_port_send('M');
    serial_port_send(' ');
    serial_port_send('1');
    serial_port_send('\r');
}

void camera_reset(void)
{   
    serial_init_command();
    serial_port_send('R');
    serial_port_send('S');
    serial_port_send('\r');
}

void camera_settings(void)
{   
    serial_init_command();
    serial_port_send('C');
    serial_port_send('R');
    serial_port_send(' ');
    serial_port_send('1');
    serial_port_send('9');
    serial_port_send(' ');
    serial_port_send('3');
    serial_port_send('2');
}

void camera_track_color(int color)
{
    char red[] = "TC 149 170 15 17 15 17\r";   /*"TC 110 140 5 25 5 25\r"*/
    char green[] = "TC 70 120 140 190 50 80\r";
    char black[] = "TC 15 40 25 35 8 22\r";
    int i;
    int length;
    
    serial_init_command();
    
    if(color == RED_COLOR)
    {
      length = strlen(red);
    
      for (i = 0; i < length; i++)
      {
    	serial_port_send(red[i]);
      }
    }
    else if(color == GREEN_COLOR)
    {
      length = strlen(green);
    
      for (i = 0; i < length; i++)
      {
    	serial_port_send(green[i]);
      }    
    }
    else if(color == BLACK_COLOR)
    {
      length = strlen(black);
    
      for (i = 0; i < length; i++)
      {
    	serial_port_send(black[i]);
      }        
    }
}

/* Handler d'interruption pour la rception de donnes sur le port srie */   
void c_int1(void)
{ 
  char val;
  
  asm("	SETC INTM"); 
  IFR = IFR | 0x1;
  
  /* On compte les interruptions (dbuggage) */
  nbInterruptionSerie = nbInterruptionSerie + 1;
  
  /* Rcupration de la valeur sur le port */
  val = serial_port_receive();
  
  /* Si c'est un ':', alors la rception de la commande est termine */
  if(val == ':')
  {
  	receptionCommandeTerminee = 1;
  }
  
  /* Sinon, ajout dans le buffer et incrmentation de la position */
  else
  {  
    serial_buffer[serial_buffer_rx_ptr] = val;
    serial_buffer_rx_ptr = serial_buffer_rx_ptr + 1;
  }
  
  asm("	CLRC INTM");                           
}

/* Verifie que la commande precedente s'est bien deroulee. 
 * Renvoie 0 si oui, -1 sinon et -2 si le rsultat de la commande est indfini
 */
static int serial_command_successful(void)
{
  if(serial_buffer[0] == 'A' && serial_buffer[1] == 'C' && serial_buffer[2] == 'K')
  {
  	return 0;
  }
  	
  if(serial_buffer[0] == 'N' && serial_buffer[1] == 'C' && serial_buffer[2] == 'K')
  {
  	return -1;
  }
  	
  return -2;
}


/* Attend la fin de la commande et renvoie le rsultat */
int serial_command_wait(void)
{        
  int ret;            
  int delay = 7000;
  
  while(receptionCommandeTerminee == 0)
  {
     int i;
      for (i = 0; i < 100; i++) { i = i + 1 ; i = i - 1; }
     delay--;
     if(delay == 0)
     {      
       return -3;
     }
  }
  
  ret = serial_command_successful();
  
  return ret;
}

int serial_check_m_packet()
{  
  int i;
  int spaces = 0;
  
  if(serial_buffer[4] != 'M' || serial_buffer[5] != ' ')
  {
    return -1;
  }
  
  for (i = 4; i < 64 ; i++)
  {
    if(serial_buffer[i] == '\r')
    {
      break;
    }
  }
  
  if(i == 63)
  {
     return -1;
  }
  
  for (i = 4; serial_buffer[i] != '\r' ; i++)
  {
     if(serial_buffer[i] == ' ')
     {
       spaces++;
     }
  }
  
  if(spaces != 8)
  {
     return -1;
  }
  
  return 0;
}

int serial_receive_m_command(int *x, int *y, int *confidence)
{  
   char buf[4];
   int i;
   int j;
   int valx, valy, valconf;
   int ret;
     
   *x = 0;
   *y = 0;
   *confidence = 0;
   
   ret = serial_check_m_packet();
   if(ret < 0)
   {
     return ret;
   }
   
   for (j = 0; j < 4 ; j++) buf[j] = 0;
   
   /* Rcupration de la coordonne 'x' */
   for (i = 6, j = 0 ; serial_buffer[i] != ' ' ; i++, j++)
   {  
      buf[j] = serial_buffer[i];
   }
   
   /* Passer le ' ' */
   i++;

   /* Conversion en entier */   
   buf[3] = 0;
   valx = atoi(buf);
   *x = valx;
   camera_barycentre_x[camera_current_record] = valx;
   
   for (j = 0; j < 4 ; j++) buf[j] = 0;
   
   /* Conversion de la coordonne 'y' */   
   for (j = 0 ; serial_buffer[i] != ' ' ; i++, j++)
   {
   	  buf[j] = serial_buffer[i];
   }

   buf[3] = 0;
   valy = atoi(buf);
   *y = valy;
   camera_barycentre_y[camera_current_record] = valy;
   
   i++;      
   
   /* Coin gauche x */
   for ( ; serial_buffer[i] != ' ' ; i++);
   i++;
   /* Coin gauche y */
   for ( ; serial_buffer[i] != ' ' ; i++);
   i++;
   /* Coin droite x */
   for ( ; serial_buffer[i] != ' ' ; i++);
   i++;
   /* Coin droite y */
   for ( ; serial_buffer[i] != ' ' ; i++);
   i++;   
   /* Nombre de pixels */
   for ( ; serial_buffer[i] != ' ' ; i++);
   i++;

   for (j = 0; j < 4 ; j++) buf[j] = 0;
   
   /* Rcupration de l'indice de confiance */
   for (j = 0 ; ((serial_buffer[i] != '\r') && (serial_buffer[i] != ' ')) ; i++, j++)
   {
      buf[j] = serial_buffer[i];
   }
   
   /* Passer le ' ' */
   i++;

   /* Conversion en entier */   
   buf[3] = 0;
   valconf = atoi(buf);   
   *confidence = valconf;   
   camera_confiance[camera_current_record] = valconf;
   
   camera_current_record++;
   
   return 0;
}
